/****************************************************************************
**  SIONLIB     http://www.fz-juelich.de/jsc/sionlib                       **
*****************************************************************************
**  Copyright (c) 2008-2019                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*!
 * \cond sion_internal
 * @file
 * @brief callback wrappers for OMP thread safe fd manipulation
 * @author David Montoya
 */

#ifndef SION_SION_LOCK_H
#define SION_SION_LOCK_H

void _lsion_par_init_debug(void);

#endif

/// \endcond
