/****************************************************************************
**  SIONLIB     http://www.fz-juelich.de/jsc/sionlib                       **
*****************************************************************************
**  Copyright (c) 2008-2019                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*!
 * \cond sion_internal
 * \file
 */

#ifndef SION_SION_LOCK_H
#define SION_SION_LOCK_H

#ifdef __cplusplus
extern "C" {
#endif

int _lsion_lock_register_lock_callbacks(int lock(void *), int unlock(void *), void *lockdata);
int _lsion_lock_user_callbacks_defined(void);
int _lsion_lock(void);
int _lsion_unlock(void);

#ifdef __cplusplus
}
#endif

#endif

/// \endcond
